% Forced damped oscillator
%  - Andy Ruina, Feb 24, 2021
% Without options.maxtime,  & without options.RelTol
% Uses tarray and not tspan

% Define parameters, the set below is interesting
% Can you explain it?
p.k = 1; p.m = 1; p.c=0;  p.F0 = 1; p.omega0 = 1.01;
x0     = 0; v0=0;  % initial conditions, ICs
z0     = [x0;v0];  % Pack ICs into z0
tarray = linspace(0, 628, 1e5);  %All of the times for the solution

%Define 'anonymous' function  EQN to pass parameters into rhs function
eqn = @(t,z)  forceddamped(t,z,p);

% Use our custom function ode2 to solve our ODE
[tarray, zarray] = ode2(eqn, tarray, z0);
x = zarray(:,1);  % Pull out the xvalues associated with tarray

plot(tarray,x);  xlabel('t');  ylabel('x')
shg


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The forced-damped oscillator ODEs
function zdot = forceddamped(t, z, p)
x = z(1);   v = z(2);  %unpack x and v
%unpack parameters
k = p.k; m= p.m; c = p.c; F0 = p.F0; omega0=p.omega0;

% Here is the F(x,v,t) for this problem
Force =  - k * x  - c * v + F0 * sin(omega0 * t);

%These are the ODEs, the same for every 1D particle problem.
xdot  = v;
vdot  = Force/m;

zdot = [xdot;vdot];  % pack ODEs into zdot
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
