function [tarray, zarray] = ODE2(eqn,tarray,z0)
% 2nd order ODE solver, called, for obvious reasons, 'The midpoint method'
% It is also called 'RK2'
% -Andy Ruina , Feb 24, 2021
% This has no error estimation and no max time.
% That is, no use of options.maxtime,  & also without options.RelTol
% Uses tarray and not tspan.


ntimes = length(tarray);     % user proves tarray, this counts the elements.
nstates   = length(z0);      % This counts the number of first order ODEs
zarray = zeros(ntimes,nstates);  % Initialize the solution matrix

zarray(1,:) = z0';  %First row of solution is the initial condition
znew = z0;           % Needed for the first time through the solution loop

    % once through for each time interva
    for i = 1:ntimes-1    %start midpoint loop
        znow     = znew;          % z at start of step
        tnow     = tarray(i);     % t at start of step
        tnew     = tarray(i+1);   % t at end of step
        h        = tnew  -  tnow; % size of time step
        thalf    = tnow + h/2;    % t , half way through a step
        
        zdotnow  = eqn(tnow,znow);  % zdot at start of step
        zhalf    = znow + (h/2) * zdotnow;   % Take half a step
        zdothalf = eqn(thalf,zhalf);      % find zdot at the half step
        znew     = znow + h * zdothalf;   % The heart of the whole thing

        zarray(i+1,:) = znew';  % put the new z into the zarray
    end % end midpoint loop

end  % end of ODE2

